# DXMIO with IMU サンプルコード

ライブラリ類の説明が貧弱なこともあって、それを読み解きながらコーディングするのはかなり面倒。以前よりも多少なりとも応用が利きそうな各I/Fを介してデバイスと通信を行うプログラムを追加した。  
またDXMIOをDynamixel等のサーボと同じネットワークに接続して使用するサンプルコードの詳細はホームページに記載した (https://www.besttechnology.co.jp/modules/knowledge/?BTE098%20DXMIO%20with%20IMU#qe68e003) 通りだが、サーボ特有の通信プロトコルを介して通信するのがこれまた煩雑なので、Windows限定だがPCをホストとした簡単なプログラムも同梱した。

# License

BestTechnology CO.,LTD. 2022  
Creative Commons Attribution ShareAlike 4.0  
https://creativecommons.org/licenses/by-sa/4.0/

他のライセンスが必要な場合は、お気軽にご連絡を。

# 同梱ファイル
```bash
.
├── bin/
│   └── *.bin  (サンプルコードのコンパイル済みバイナリファイル)
├── src/
│   ├── STSW-IMG007 (VL53L1X API)
│   │   ├── API/
│   │   │   ├ core/
│   │   │   ├ platform/
│   │   │   ├ libIMG007.a
│   │   │   ├ makefile
│   │   │   └ makelib.cmd
│   │   └── DOC/
│   ├── STSW-IMG009 (VL53L1X ULD API)
│   │   ├ core/
│   │   ├ platform/
│   │   ├ libIMG009.a
│   │   ├ makefile
│   │   └ makelib.cmd
│   ├── smpl*.c  (サンプルコード)
│   ├── smpl*.cpp  (サンプルコード)
│   ├── e097b_*.h
│   ├── *.raw
│   ├── makebin.cmd
│   ├── makebin_E186.cmd
│   └── makefile
├── tools/
│   ├── sterm/
│   │   ├── STERM.exe  (Simple Term本体)
│   │   ├── STERM_LPC84x_WriteAndGo.txt  (転送スクリプト)
│   │   └── conf.cmd  (簡易実行用バッチ)
│   ├── dxl/
│   │   ├── cui/
│   │   │   ├── dxmio2_dxl.exe (DXL用CUI版簡易ホスト)
│   │   │   ├── dxmio2_dxl.c
│   │   │   ├── build_exe.cmd
│   │   │   ├── *.h
│   │   │   └── *.dll
│   │   └ gui/
│   │       ├── DXCONF32.exe (DXL用GUI版ホスト)
│   │       └── *.dll
│   ├── b3m/
│   │   ├── dxmio2_b3m.exe (B3M用CUI版簡易ホスト)
│   │   ├── dxmio2_b3m.c
│   │   ├── build_exe.cmd
│   │   ├── *.h
│   │   └── *.dll
│   └── ics/
│       ├── dxmio2_ics.exe (ICS用CUI版簡易ホスト)
│       ├── dxmio2_ics.c
│       ├── build_exe.cmd
│       ├── *.h
│       └── *.dll
└── README.md
```
# 事前の準備

2022年4月以降に公開されたGCC Developer Liteが必要。  
https://www.besttechnology.co.jp/modules/knowledge/?GCC%20Developer%20Lite

「基本パック」「ARMパック」をダウンロードしインストールする。Windows上で動作するプログラムのソースコードをコンパイルする場合は「WINパック」も必要。

# サンプルコード

srcディレクトリにCないしC++のソースを同梱し、ある程度ファイル名で判断できるように命名したつもり。  
DXMIO単体で試せるものの他に、GPIO端子に接続したセンサ等を使用するものも含まれているので、詳細はソース中のコメントを参照の事。

## サンプルコードに共通するファイル

各ソースにある程度共通する部分をヘッダファイルにして再利用している。

- e097b_def.h  
GPIOの初期条件・LED・自動初期化処理等が含まれる。  
DXMIO固有のUSARTやLED類は初期化条件を変更する必要がないので、ソースにincludeするだけで強制的に初期化が行われる。C++で有用なクラスの初期化についても、このファイルに初期化処理が含まれる。  
またお遊びでバイナリファイル等を自身のコードに直接埋め込むのが容易になるINCBINマクロはこのファイルに宣言されている。

- e097b_usart.h  
DXMIO用I/Oボードが登場した事によりUSARTの扱いが煩雑になったため、いくつかの選択肢から任意のUSART関数を選べるようにマクロを記述してある。  
先にUSART_TYPEに任意の値を設定するマクロを記述した後に本ヘッダをincludeする必要がある。  
```cpp
// USART_TYPEマクロで使用するUSARTと機能を選択
// CN3/CN4     1:ポーリング, 2:割り込み, 3:DMA
// CN1-6,CN1-7 4:ポーリング, 5:割り込み, 6:DMA
#ifndef USART_TYPE
  #define USART_TYPE 1
#endif
#include  "e097b_usart.h"
```
- e097b_imu.h  
DXMIOに搭載されたIMU(BNO055)へアクセスするサブルーチンやレジスタのアドレスを記載したマクロが宣言されている。  
I2CバスでIMUと通信する構成のため、I2Cライブラリを使用している。

- *.raw  
 11.025kHzでリサンプリングした生音データ。  
「smpl12(dac_synth).cpp」や「smpl13(dac_music).cpp」のサンプルソース内でINCBINマクロを使ってそれらファイルを直接リソースへ取り込んで使用している。

- src/STSW-IMG007, src/STSW-IMG009  
ST Microelectronics社のToFセンサのAPIが含まれており、DXMIO向けにI/Fを追加した上でライブラリ化してあるが、今のところSTSW-IMG007をsmpl8(i2c_LV53L1X).cのみで使っている。

- makebin.cmd, makebin_E186.cmd, makefile  
GCC Developer Liteを起動せずともコンパイルするためのmakefileと、コンパイラへのパスを設定してmakeを起動するバッチファイル。

# DXMIO用サンプルコードのコンパイル

ホームページに記載されている通りにGCC Developre Liteでサンプルコードを開いて諸設定を行った上でコンパイルするが、ToFを対象にしたサンプルコードのみ追加のライブラリを要求するので、その場合は適宜コンパイルオプションを変更する必要がある。  
その代わりにsrcフォルダに同梱した「makebin.cmd」を単体で実行する事で、全サンプルコードを対象にコンパイルを行ってくれる。コンパイルが成功するとbinフォルダにコンパイル済みのバイナリファイルが生成される。また任意の1つのソースコードをmakebin.cmdにドロップダウンすると、そのソースコードだけをコンパイルできる。  
「makebin_E186.cmd」はDXMIO用I/Oボードを使用している場合に、I/OボードのUSBシリアルポートをコンソールにするためのマクロを定義し直してコンパイルを行う。

なおコンパイル済みのオブジェクトやbinファイルが存在する状態では再構築されないため、その場合はbinとobjフォルダを予め削除してからコンパイルする事。

# DXMIOへの書き込み

こちらもDXMIOのホームページに記載されている通りだが、本サンプルにも書き込みに使用するSTERM.exeをtools/stermフォルダに同梱。GCC Developer Liteに同梱されているものに対してスクリプトファイルと起動用のバッチファイルconf.cmdを追加してある。

conf.cmdにコンパイル済みのバイナリファイルをドロップダウンすると、転送に必要な設定がなされた状態でSTERM.exeが起動するように仕掛けてある。準備が整ったら「スクリプト実行」で転送を開始し、フラッシュの完全消去->転送->実行までを一括で行ってくれる。

# 出荷時に書き込まれるプログラム

「smpl19(dxclient_imu).c」をコンパイルして生成されるbinファイルが書き込まれている。

なおデフォルトではDynamixelプロトコルV2の設定になっているが、プロトコルの切り替えはソースの冒頭に記述されたマクロを1に書き換えてコンパイルする事でプロトコルV1に対応したbinファイルが生成される。
```cpp
// DXプロトコルバージョン選択 (1 or 2)
#define DX_PROTOCOL_VER 1
```

# IMUを使ったサンプルコードのテストツール

smpl19~smpl21が各社のサーボがサポートする通信プロトコルに対応させたIMU用のサンプルコードになる。いずれもクライアントとして機能するため別途ホストとなるものが必要となるが、それらプロトコルに勝手に対応させているので各社から対応ツールが用意されている訳ではない。  
このサンプルに含まれるホストツールは横着した作りだが、いずれもCOMポート・ボーレート・ID番号を設定した上でDXMIOのコントロールテーブルへアクセスする最低限の手段を示すものである。
- Dynamixel版ホストツール(tools/dxlフォルダ)  
GUI版とCUI版を用意。GUI版はかつて弊社で提供していたDyanmixel ConfiguratorにDXMIOを追加したものとなる。  
CUI版はプロトコルを簡易的に扱うDLLを使用しており、GCC Developer LiteにWINパックを追加している状態であれば、build_exe.cmdにてコンパイルできる。
- B3M/ICS版ホストツール(tools/b3m、tools/ICSフォルダ)  
こちらはCUI版のみとなり、同様にプロトコルを簡易的に扱うDLLを使用しており、GCC Developer LiteにWINパックを追加している状態であればbuild_exe.cmdにてコンパイルできる。
